/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2003 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.conf.impl;

import java.security.acl.*;
import java.util.*;
import org.apache.log4j.Logger;
import org.apache.commons.collections.*;
import com.silverwrist.dynamo.db.NamespaceCache;
import com.silverwrist.dynamo.db.UserManagement;
import com.silverwrist.dynamo.iface.*;
import com.silverwrist.dynamo.except.*;
import com.silverwrist.dynamo.security.SecurityReferenceMonitor;
import com.silverwrist.dynamo.util.*;
import com.silverwrist.venice.conf.ConfNamespaces;
import com.silverwrist.venice.conf.iface.*;
import com.silverwrist.venice.conf.obj.*;

class ConferenceImpl implements VeniceConference
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static Logger logger = Logger.getLogger(ConferenceImpl.class);

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private DynamicImplConference m_dobj;     // used to implement DynamicObject methods
  private ConferenceOps m_ops;              // conference database operations
  private NamespaceCache m_nscache;         // namespace cache object
  private SecurityReferenceMonitor m_srm;   // security reference monitor
  private UserManagement m_users;           // user management object
  private int m_id;                         // conference ID
  private java.util.Date m_createdate;      // creation date
  private java.util.Date m_lastupdate;      // last update date
  private int m_hosts_gid;                  // GID of community hosts group
  private int m_aclid;                      // ACL ID
  private String m_name;                    // conference name
  private ReferenceMap m_properties;        // properties cache
  private String m_prime_alias = null;      // primary alias

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  ConferenceImpl(ConferenceOps ops, NamespaceCache nscache, SecurityReferenceMonitor srm, UserManagement users,
		 Map params)
  {
    m_dobj = new DynamicImplConference(this);
    m_ops = ops;
    m_nscache = nscache;
    m_srm = srm;
    m_users = users;
    m_id = ((Integer)(params.get(ConferenceManagerOps.KEY_CONFID))).intValue();
    m_createdate = (java.util.Date)(params.get(ConferenceManagerOps.KEY_CREATE_DATE));
    m_lastupdate = (java.util.Date)(params.get(ConferenceManagerOps.KEY_LAST_UPDATE));
    m_hosts_gid = ((Integer)(params.get(ConferenceManagerOps.KEY_HOSTS_GID))).intValue();
    m_aclid = ((Integer)(params.get(ConferenceManagerOps.KEY_ACLID))).intValue();
    m_name = (String)(params.get(ConferenceManagerOps.KEY_NAME));
    m_properties = new ReferenceMap(ReferenceMap.HARD,ReferenceMap.SOFT);

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Internal operations
   *--------------------------------------------------------------------------------
   */

  /**
   * Performs a permission check on the conference's ACL, as well as the global ACL.  Returns without
   * incident if the permission check succeeds; throws a
   * {@link com.silverwrist.dynamo.except.DynamoSecurityException DynamoSecurityException} if it fails.
   *
   * @param caller The user making the call, to test the permission.
   * @param perm_namespace The namespace of the permission to be tested.
   * @param perm_name The name of the permission to be tested.
   * @param fail_message The message to be thrown as a <CODE>DynamoSecurityException</CODE> if the test fails.
   *                     Interpreted as a message identifier within the <CODE>ConferenceMessages</CODE> resource
   *                     bundle, with one parameter: the name of the conference.
   * @exception com.silverwrist.dynamo.except.DatabaseException If there was a database failure testing the permission.
   * @exception com.silverwrist.dynamo.except.DynamoSecurityException If the permission test failed.
   */
  private final void testPermission(DynamoUser caller, String perm_namespace, String perm_name,
				    String fail_message) throws DatabaseException, DynamoSecurityException
  {
    try
    { // perform security tests...
      if (caller.equals(m_srm.getAdminUser()))
	return;  // Administrator can do anything
      if (m_srm.getAcl(m_aclid).testPermission(caller,perm_namespace,perm_name))
	return;  // we have the right permission in the conference ACL
      if (m_srm.getGlobalAcl().testPermission(caller,perm_namespace,perm_name))
	return;  // or we can be authorized by the global ACL

    } // end try
    catch (AclNotFoundException e)
    { // just fall through and throw the exception
    } // end catch

    // throw the DynamoSecurityException
    DynamoSecurityException d = new DynamoSecurityException(ConferenceImpl.class,"ConferenceMessages",fail_message);
    d.setParameter(0,m_name);
    throw d;

  } // end testPermission

  /*--------------------------------------------------------------------------------
   * Overrides from class Object
   *--------------------------------------------------------------------------------
   */

  /**
   * Indicates whether some other object is "equal to" this one.
   *
   * @param o The reference object with which to compare.
   * @return <CODE>true</CODE> if this object is the same as the <CODE>o</CODE> argument; <CODE>false</CODE> otherwise.
   */
  public boolean equals(Object o)
  {
    if (o==null)
      return false;
    if (o instanceof VeniceConference)
      return (((VeniceConference)o).getConfID()==m_id);
    return false;

  } // end equals

  /**
   * Returns a hash code value for the object. This method is supported for the benefit of hashtables such as those
   * provided by {@link java.util.Hashtable Hashtable}.
   *
   * @return A hash code value for this object.
   */
  public int hashCode()
  {
    return m_id;

  } // end hashCode

  /**
   * Returns a string representation of the object. In general, the <CODE>toString</CODE> method returns a string
   * that "textually represents" this object. 
   *
   * @return A string representation of the object.
   */
  public String toString()
  {
    return "Conference \"" + m_name + "\"";

  } // end toString

  /*--------------------------------------------------------------------------------
   * Implementations from interface DynamicObject
   *--------------------------------------------------------------------------------
   */

  public Object get(String property_name) throws DynamicObjectException
  {
    return m_dobj.get(property_name);

  } // end get

  public void set(String property_name, Object value) throws DynamicObjectException
  {
    m_dobj.set(property_name,value);

  } // end set

  public Object call(String method_name, Object[] parameters) throws DynamicObjectException
  {
    return m_dobj.call(method_name,parameters);

  } // end call

  public Object call(String method_name, List parameters) throws DynamicObjectException
  {
    return m_dobj.call(method_name,parameters);

  } // end call

  public DynamicClass getDClass()
  {
    return m_dobj.getDClass();

  } // end getDClass
  
  /*--------------------------------------------------------------------------------
   * Implementations from interface NamedObject
   *--------------------------------------------------------------------------------
   */

  /**
   * Returns the name of this object.
   *
   * @return The name of this object.
   */
  public String getName()
  {
    return m_name;

  } // end getName

  /*--------------------------------------------------------------------------------
   * Implementations from interface ObjectProvider
   *--------------------------------------------------------------------------------
   */

  /**
   * Retrieves an object from this <CODE>ObjectProvider</CODE>.
   *
   * @param namespace The namespace to interpret the name relative to.
   * @param name The name of the object to be retrieved.
   * @return The object reference specified.
   */
  public Object getObject(String namespace, String name)
  {
    if (logger.isDebugEnabled())
      logger.debug("Conference.getObject: namespace = " + namespace + ", name = " + name);
    try
    { // convert the namespace name to an ID here
      PropertyKey key = new PropertyKey(m_nscache.namespaceNameToId(namespace),name);
      Object rc = null;
      synchronized (this)
      { // start by looking in the properties map
	rc = m_properties.get(key);
	if (rc==null)
	{ // no use - need to try the database
	  rc = m_ops.getProperty(m_id,key);
	  if (rc!=null)
          { // found in the database
	    m_properties.put(key,rc);
	    logger.debug("value found in database");

	  } // end if

	} // end if
	else
	  logger.debug("value found in cache");

      } // end synchronized block

      if (rc==null)
      { // the object was not found
	logger.debug("value not found");
	throw new NoSuchObjectException(this.toString(),namespace,name);

      } // end if

      return rc;

    } // end try
    catch (DatabaseException e)
    { // translate into our NoSuchObjectException but retain the DatabaseException
      logger.debug("Database exception while doing find",e);
      throw new NoSuchObjectException(this.toString(),namespace,name,e);

    } // end catch

  } // end getObject

  /*--------------------------------------------------------------------------------
   * Implementations from interface SecureObjectStore
   *--------------------------------------------------------------------------------
   */

  /**
   * Sets an object into this <CODE>SecureObjectStore</CODE>.
   *
   * @param caller The user performing the operation.
   * @param namespace The namespace to interpret the name relative to.
   * @param name The name of the object to be set.
   * @param value The object to set into the <CODE>SecureObjectStore</CODE>.
   * @return The previous object that was set into the <CODE>SecureObjectStore</CODE> under this namespace and name, or
   *         <CODE>null</CODE> if there was no such object.
   * @exception com.silverwrist.dynamo.except.DatabaseException If there was an error setting the object value.
   * @exception com.silverwrist.dynamo.except.DynamoSecurityException If the specified user is not permitted to
   *            set this object value into this <CODE>SecureObjectStore</CODE>.
   */
  public Object setObject(DynamoUser caller, String namespace, String name, Object value)
      throws DatabaseException, DynamoSecurityException
  {
    testPermission(caller,namespace,"set.property","auth.setProperty");
    DateObjectPair rc = null;

    // convert the namespace name to an ID here
    PropertyKey key = new PropertyKey(m_nscache.namespaceNameToId(namespace),name);
    synchronized (this)
    { // start by setting the database value
      rc = m_ops.setProperty(m_id,key,value);
      
      // and cache it, too
      m_properties.put(key,value);

      // and save off the update date/time
      m_lastupdate = rc.getDate();

    } // end synchronized block

    // TODO: m_post.postUpdate(new CommunityPropertyUpdateEvent(this,namespace,name));
    return rc.getObject();

  } // end setObject

  /**
   * Removes an object from this <CODE>SecureObjectStore</CODE>.
   *
   * @param caller The user performing the operation.
   * @param namespace The namespace to interpret the name relative to.
   * @param name The name of the object to be removed.
   * @return The previous object that was set into the <CODE>SecureObjectStore</CODE> under this namespace and name, or
   *         <CODE>null</CODE> if there was no such object.
   * @exception com.silverwrist.dynamo.except.DatabaseException If there was an error removing the object value.
   * @exception com.silverwrist.dynamo.except.DynamoSecurityException If the specified user is not permitted to
   *            remove this object value from this <CODE>SecureObjectStore</CODE>.
   */
  public Object removeObject(DynamoUser caller, String namespace, String name)
      throws DatabaseException, DynamoSecurityException
  {
    testPermission(caller,namespace,"remove.property","auth.removeProperty");
    DateObjectPair rc = null;

    // convert the namespace name to an ID here
    PropertyKey key = new PropertyKey(m_nscache.namespaceNameToId(namespace),name);
    synchronized (this)
    { // start by killing the database value
      rc = m_ops.removeProperty(m_id,key);

      // and remove the cached value, too
      m_properties.remove(key);

      if (rc!=null)  // save off update date.time
	m_lastupdate = rc.getDate();

    } // end synchronized block

    // TODO: m_post.postUpdate(new CommunityPropertyUpdateEvent(this,namespace,name));
    return (rc==null) ? null : rc.getObject();

  } // end removeObject

  /**
   * Returns a collection of all object namespaces that have been set into this <CODE>SecureObjectStore</CODE>.
   *
   * @return A {@link java.util.Collection Collection} containing {@link java.lang.String String} objects specifying
   *         all the object namespaces.
   * @exception com.silverwrist.dynamo.except.DatabaseException If there was an error getting the namespace list.
   */
  public Collection getNamespaces() throws DatabaseException
  {
    // call through to the database to get the list of namespace IDs
    int[] ids = m_ops.getPropertyNamespaceIDs(m_id);

    ArrayList rc = new ArrayList(ids.length);
    for (int i=0; i<ids.length; i++)
      rc.add(m_nscache.namespaceIdToName(ids[i]));
    return Collections.unmodifiableList(rc);

  } // end getNamespaces

  /**
   * Returns a collection of all object names that have been set into this <CODE>SecureObjectStore</CODE> under
   * a given namespace.
   *
   * @param namespace The namespace to look for names under.
   * @return A {@link java.util.Collection Collection} containing {@link java.lang.String String} objects
   *         specifying all the object names for this namespace.
   * @exception com.silverwrist.dynamo.except.DatabaseException If there was an error getting the object name list.
   */
  public Collection getNamesForNamespace(String namespace) throws DatabaseException
  {
    // call through to the database to get the data for this namespace
    int nsid = m_nscache.namespaceNameToId(namespace);
    Map data = m_ops.getAllProperties(m_id,nsid);

    // we both create the return value and cache the data values
    ArrayList rc = new ArrayList(data.size());
    synchronized (this)
    { // do the transfer...
      Iterator it = data.entrySet().iterator();
      while (it.hasNext())
      { // copy one entry at a time
	Map.Entry ntry = (Map.Entry)(it.next());
	rc.add(ntry.getKey().toString());
	m_properties.put(new PropertyKey(nsid,ntry.getKey().toString()),ntry.getValue());

      } // end while

    } // end synchronized block

    return Collections.unmodifiableList(rc);

  } // end getNamesForNamespace

  /*--------------------------------------------------------------------------------
   * Implementations from interface VeniceConference
   *--------------------------------------------------------------------------------
   */

  public int getConfID()
  {
    return m_id;

  } // end getConfID

  public void setName(DynamoUser caller, String name) throws DatabaseException, DynamoSecurityException
  {
    testPermission(caller,ConfNamespaces.PERMISSIONS_NAMESPACE,"set.name","auth.setName");
    synchronized (this)
    { // update the name
      java.util.Date updated = m_ops.setName(m_id,name);
      m_name = name;
      m_lastupdate = updated;

    } // end synchronized block

  } // end setName

  public java.util.Date getCreatedDate()
  {
    return m_createdate;

  } // end getCreatedDate

  public java.util.Date getLastUpdateDate()
  {
    return m_lastupdate;

  } // end getLastUpdateDate

  public synchronized void setLastUpdateDate(DynamoUser caller, java.util.Date date)
      throws DatabaseException, DynamoSecurityException
  {
    m_ops.setLastUpdateDate(m_id,date);
    m_lastupdate = date;

  } // end setLastUpdateDate

  public int getHostsGID()
  {
    return m_hosts_gid;

  } // end getHostsGID

  public DynamoGroup getHosts() throws DatabaseException
  {
    return m_users.getGroup(m_hosts_gid);

  } // end getHosts

  public DynamoAcl getAcl() throws DatabaseException, AclNotFoundException
  {
    return m_srm.getAcl(m_aclid);

  } // end getAcl

  public Set getAliases() throws DatabaseException
  {
    return m_ops.getAliases(m_id);

  } // end getAliases

  public synchronized String getPrimaryAlias() throws DatabaseException
  {
    if (m_prime_alias==null)
      m_prime_alias = m_ops.getPrimaryAlias(m_id);
    return m_prime_alias;

  } // end getPrimaryAlias

  public void setPrimaryAlias(DynamoUser caller, String alias) throws DatabaseException, DynamoSecurityException
  {
    testPermission(caller,ConfNamespaces.PERMISSIONS_NAMESPACE,"set.alias","auth.setAlias");
    synchronized (this)
    { // change the prime alias
      java.util.Date update = m_ops.setPrimaryAlias(m_id,alias);
      m_prime_alias = alias;
      if (update!=null)
	m_lastupdate = update;

    } // end synchronized block

  } // end setPrimaryAlias

  public void addAlias(DynamoUser caller, String alias) throws DatabaseException, DynamoSecurityException
  {
    testPermission(caller,ConfNamespaces.PERMISSIONS_NAMESPACE,"set.alias","auth.setAlias");
    synchronized (this)
    { // add an alias
      java.util.Date update = m_ops.addAlias(m_id,alias);
      if (update!=null)
	m_lastupdate = update;

    } // end synchronized block

  } // end addAlias

  public void removeAlias(DynamoUser caller, String alias) throws DatabaseException, DynamoSecurityException
  {
    testPermission(caller,ConfNamespaces.PERMISSIONS_NAMESPACE,"set.alias","auth.setAlias");
    synchronized (this)
    { // remove the alias
      java.util.Date update = m_ops.removeAlias(m_id,alias);
      if (update!=null)
	m_lastupdate = update;

    } // end synchronized block

  } // end removeAlias

} // end class ConferenceImpl
