/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2003 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.unistore;

import java.sql.*;
import java.util.*;
import com.silverwrist.util.*;
import com.silverwrist.dynamo.except.*;
import com.silverwrist.dynamo.iface.*;
import com.silverwrist.dynamo.util.*;

class TextPartOps_mysql extends TextPartOps
{
  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private PropertySerializer m_psz;        // reference to property serializer
  private DBUtilities m_utils;

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  TextPartOps_mysql(DBConnectionPool pool)
  {
    super(pool);
    m_psz = (PropertySerializer)(pool.queryService(PropertySerializer.class));
    m_utils = (DBUtilities)(pool.queryService(DBUtilities.class));

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Overrides from class OpsBase
   *--------------------------------------------------------------------------------
   */

  public void dispose()
  {
    m_psz = null;
    m_utils = null;
    super.dispose();

  } // end dispose

  /*--------------------------------------------------------------------------------
   * Abstract implementations from class TextPartOps
   *--------------------------------------------------------------------------------
   */

  Object getProperty(long msgid, int part, PropertyKey key) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rs = null;
    String rc_str = null;
    try
    { // get a connection
      conn = getConnection();

      // look up the property
      stmt = conn.prepareStatement("SELECT prop_value FROM us_text_prop WHERE msgid = ? AND part = ? AND nsid = ? "
				   + "AND prop_name = ?;");
      stmt.setLong(1,msgid);
      stmt.setInt(2,part);
      stmt.setInt(3,key.getNamespaceID());
      stmt.setString(4,key.getName());
      rs = stmt.executeQuery();
      if (!(rs.next()))
	return null;  // property not found

      rc_str = rs.getString(1);

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(conn);

    } // end finally

    // Deserialize the property value.
    Object rc = m_psz.deserializeProperty(rc_str);
    if (rc!=null)
      return rc;

    // deserialization exception - throw it
    DatabaseException de = new DatabaseException(TextPartOps_mysql.class,"UniStoreMessages","property.deserialize");
    de.setParameter(0,key.getName());
    throw de;

  } // end getProperty

  Object setProperty(long msgid, int part, PropertyKey key, Object value) throws DatabaseException
  {
    String serialized_value = m_psz.serializeProperty(value);
    if (serialized_value==null)
    { // serialization exception - throw it
      DatabaseException de = new DatabaseException(TextPartOps_mysql.class,"UniStoreMessages","property.serialize");
      de.setParameter(0,key.getName());
      throw de;

    } // end if

    String old_value = null;
    Connection conn = null;
    PreparedStatement stmt = null;
    Statement stmt2 = null;
    ResultSet rs = null;
    try
    { // get a connection
      conn = getConnection();

      // lock the table
      stmt2 = conn.createStatement();
      stmt2.executeUpdate("LOCK TABLES us_text_prop WRITE;");

      // look to see if the property value is already there
      stmt = conn.prepareStatement("SELECT prop_value FROM us_text_prop WHERE msgid = ? AND part = ? AND nsid = ? "
				   + "AND prop_name = ?;");
      stmt.setLong(1,msgid);
      stmt.setInt(2,part);
      stmt.setInt(3,key.getNamespaceID());
      stmt.setString(4,key.getName());
      rs = stmt.executeQuery();
      if (rs.next())
	old_value = rs.getString(1);
      SQLUtils.shutdown(rs);
      rs = null;
      SQLUtils.shutdown(stmt);

      if (old_value!=null)
      { // prepare the statement to update the existing record
	stmt = conn.prepareStatement("UPDATE us_text_prop SET prop_value = ? WHERE msgid = ? AND part = ? "
				     + "AND nsid = ? AND prop_name = ?;");
	stmt.setString(1,serialized_value);
	stmt.setLong(2,msgid);
	stmt.setInt(3,part);
	stmt.setInt(4,key.getNamespaceID());
	stmt.setString(5,key.getName());

      } // end if
      else
      { // prepare the statement to insert a new record
	stmt = conn.prepareStatement("INSERT INTO us_text_prop (msgid, part, nsid, prop_name, prop_value) "
				     + "VALUES (?, ?, ?, ?, ?);");
	stmt.setLong(1,msgid);
	stmt.setInt(2,part);
	stmt.setInt(3,key.getNamespaceID());
	stmt.setString(4,key.getName());
	stmt.setString(5,serialized_value);

      } // end else

      stmt.executeUpdate();  // execute it!

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      MySQLUtils.unlockTables(conn);
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(stmt2);
      SQLUtils.shutdown(conn);

    } // end finally

    if (old_value==null)
      return null;  // no previous value

    // Deserialize the property value.
    Object rc = m_psz.deserializeProperty(old_value);
    if (rc!=null)
      return rc;

    // deserialization exception - throw it
    DatabaseException de = new DatabaseException(TextPartOps_mysql.class,"UniStoreMessages","property.deserialize");
    de.setParameter(0,key.getName());
    throw de;

  } // end setProperty

  Object removeProperty(long msgid, int part, PropertyKey key) throws DatabaseException
  {
    String old_value = null;
    Connection conn = null;
    PreparedStatement stmt = null;
    Statement stmt2 = null;
    ResultSet rs = null;
    try
    { // get a connection
      conn = getConnection();

      // lock the table
      stmt2 = conn.createStatement();
      stmt2.executeUpdate("LOCK TABLES us_text_prop WRITE;");

      // look to see if the property value is already there
      stmt = conn.prepareStatement("SELECT prop_value FROM us_text_prop WHERE msgid = ? AND part = ? AND nsid = ? "
				   + "AND prop_name = ?;");
      stmt.setLong(1,msgid);
      stmt.setInt(2,part);
      stmt.setInt(3,key.getNamespaceID());
      stmt.setString(4,key.getName());
      rs = stmt.executeQuery();
      if (rs.next())
	old_value = rs.getString(1);
      else
	return null;  // no need to remove anything
      SQLUtils.shutdown(rs);
      rs = null;
      SQLUtils.shutdown(stmt);

      // delete the database row
      stmt = conn.prepareStatement("DELETE FROM us_text_prop WHERE msgid = ? AND part = ? AND nsid = ? "
				   + "AND prop_name = ?;");
      stmt.setLong(1,msgid);
      stmt.setInt(2,part);
      stmt.setInt(3,key.getNamespaceID());
      stmt.setString(4,key.getName());
      stmt.executeUpdate();

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      MySQLUtils.unlockTables(conn);
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(stmt2);
      SQLUtils.shutdown(conn);

    } // end finally

    // Deserialize the property value.
    Object rc = m_psz.deserializeProperty(old_value);
    if (rc!=null)
      return rc;

    // deserialization exception - throw it
    DatabaseException de = new DatabaseException(TextPartOps_mysql.class,"UniStoreMessages","property.deserialize");
    de.setParameter(0,key.getName());
    throw de;

  } // end removeProperty

  int[] getPropertyNamespaceIDs(long msgid, int part) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rs = null;
    try
    { // get a connection
      conn = getConnection();

      // execute the query!
      stmt = conn.prepareStatement("SELECT DISTINCT nsid FROM us_text_prop WHERE msgid = ? AND part = ?;");
      stmt.setLong(1,msgid);
      stmt.setInt(2,part);
      rs = stmt.executeQuery();

      // read out a list of the namespace IDs
      ArrayList tmp = new ArrayList();
      while (rs.next())
	tmp.add(new Integer(rs.getInt(1)));

      // create and return the array
      int[] rc = new int[tmp.size()];
      for (int i=0; i<tmp.size(); i++)
	rc[i] = ((Integer)(tmp.get(i))).intValue();
      tmp.clear();
      return rc;

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(conn);

    } // end finally

  } // end getPropertyNamespaceIDs

  Map getAllProperties(long msgid, int part, int namespace) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rs = null;
    try
    { // get a connection
      conn = getConnection();

      // execute the query!
      stmt = conn.prepareStatement("SELECT prop_name, prop_value FROM us_text_prop WHERE msgid = ? AND part = ? "
				   + "AND nsid = ?;");
      stmt.setLong(1,msgid);
      stmt.setInt(2,part);
      stmt.setInt(3,namespace);
      rs = stmt.executeQuery();

      // prepare the return value
      HashMap rc = new HashMap();
      while (rs.next())
      { // copy data out, deserializing properties as we go
	String key = rs.getString(1);
	Object value = m_psz.deserializeProperty(rs.getString(2));
	if (value==null)
	{ // deserialization exception - throw it
	  DatabaseException de = new DatabaseException(TextPartOps_mysql.class,"UniStoreMessages",
                                                       "property.deserialize");
	  de.setParameter(0,key);
	  throw de;

	} // end if

	rc.put(key,value);

      } // end while

      return rc;

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(conn);

    } // end finally

  } // end getAllProperties

  java.util.Date touchRead(long msgid, int part) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    Statement stmt2 = null;
    ResultSet rs = null;
    try
    { // get a connection
      conn = getConnection();

      // lock the table
      stmt2 = conn.createStatement();
      stmt2.executeUpdate("LOCK TABLES us_text WRITE;");

      // prepare and execute the statement
      stmt = conn.prepareStatement("UPDATE us_text SET reads = reads + 1, lastread = ? WHERE msgid = ? AND part = ?;");
      java.util.Date rc = new java.util.Date();
      m_utils.setDateTime(stmt,1,rc);
      stmt.setLong(2,msgid);
      stmt.setInt(3,part);
      stmt.executeUpdate();
      return rc;

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      MySQLUtils.unlockTables(conn);
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(stmt2);
      SQLUtils.shutdown(conn);

    } // end finally

  } // end touchRead

  String getText(long msgid, int part) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rs = null;
    try
    { // get a connection
      conn = getConnection();

      // create and execute the statement
      stmt = conn.prepareStatement("SELECT data FROM us_text WHERE msgid = ? AND part = ?;");
      stmt.setLong(1,msgid);
      stmt.setInt(2,part);
      rs = stmt.executeQuery();
      if (!(rs.next()))
      { // could not find the part
	DatabaseException de = new DatabaseException(TextPartOps_mysql.class,"UniStoreMessages","bad.loadText.part");
	de.setParameter(0,String.valueOf(part));
	de.setParameter(1,String.valueOf(msgid));
	throw de;

      } // end if

      return rs.getString(1);

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      SQLUtils.shutdown(rs);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(conn);

    } // end finally

  } // end getText

  void delete(long msgid, int part) throws DatabaseException
  {
    Connection conn = null;
    PreparedStatement stmt = null;
    Statement stmt2 = null;
    try
    { // get a connection
      conn = getConnection();

      // lock the tables
      stmt2 = conn.createStatement();
      stmt2.executeUpdate("LOCK TABLES us_text WRITE, us_text_props WRITE;");

      // delete all entries from the main and property tables
      stmt = conn.prepareStatement("DELETE FROM us_text WHERE msgid = ? AND part = ?;");
      stmt.setLong(1,msgid);
      stmt.setInt(2,part);
      stmt.executeUpdate();
      stmt.close();
      stmt = conn.prepareStatement("DELETE FROM us_text_prop WHERE msgid = ? AND part = ?;");
      stmt.setLong(1,msgid);
      stmt.setInt(2,part);
      stmt.executeUpdate();
      stmt.close();

      // renumber all entries above that one to close the gap
      stmt = conn.prepareStatement("UPDATE us_text SET part = part - 1 WHERE msgid = ? AND part > ?;");
      stmt.setLong(1,msgid);
      stmt.setInt(2,part);
      stmt.executeUpdate();
      stmt.close();
      stmt = conn.prepareStatement("UPDATE us_text_prop SET part = part - 1 WHERE msgid = ? AND part > ?;");
      stmt.setLong(1,msgid);
      stmt.setInt(2,part);
      stmt.executeUpdate();

    } // end try
    catch (SQLException e)
    { // translate to a general DatabaseException
      throw generalException(e);

    } // end catch
    finally
    { // shut everything down
      MySQLUtils.unlockTables(conn);
      SQLUtils.shutdown(stmt);
      SQLUtils.shutdown(stmt2);
      SQLUtils.shutdown(conn);

    } // end finally

  } // end delete

} // end class TextPartOps_mysql
